pub enum Error {
Disconnected,
Interrupted,
}Expand description
An error related to the actor system
Variants
Disconnected
The actor is no longer running and disconnected from the sending address.
Interrupted
The message request operation was interrupted. This happens when the message result sender
is dropped. Therefore, it should never be returned from detached SendFutures
This could be due to the actor’s event loop being shut down, or due to a custom timeout.
Unlike Error::Disconnected, it does not necessarily imply that any retries or further
attempts to interact with the actor will result in an error.